package co.jp.mamol.myapp.controller;

import co.jp.mamol.myapp.common.ApiResponse;
import co.jp.mamol.myapp.form.BuyApprovalForm;
import co.jp.mamol.myapp.model.SizaiModel;
import co.jp.mamol.myapp.model.UserModel;
import co.jp.mamol.myapp.service.BuyApprovalService;
import jakarta.servlet.http.HttpSession;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;

@RestController
@RequestMapping("/api")
public class BuyApprovalController {

    private final BuyApprovalService buyApprovalService;

    @Autowired
    public BuyApprovalController(BuyApprovalService buyApprovalService) {
        this.buyApprovalService = buyApprovalService;
    }

    private UserModel getLoginUser(HttpSession session) {

        return (UserModel) session.getAttribute("loginInfo");
    }

    // M0004,BL2001,DA2001
    @GetMapping("/approvalList/init")
    public ResponseEntity<ApiResponse<BuyApprovalForm>> init(HttpSession session){
        UserModel user = getLoginUser(session);
        if (user == null || !"2".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        LocalDate endDate = LocalDate.now();
        LocalDate startDate = endDate.minusMonths(1);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String startDateStr = startDate.format(formatter);
        String endDateStr = endDate.format(formatter);

        List<SizaiModel> sizaiList = buyApprovalService.getDeptRequestList(startDateStr, endDateStr, user.getDepid());
        BuyApprovalForm buyApprovalForm = new BuyApprovalForm();
        if (sizaiList == null || sizaiList.isEmpty()) {

            return ResponseEntity.badRequest().body(ApiResponse.fail("指定期間の購入依頼が登録されていません"));
        } else {
            buyApprovalForm.setSizaiList(sizaiList);
        }
        buyApprovalForm.setStartDate(startDateStr);
        buyApprovalForm.setEndDate(endDateStr);
        return ResponseEntity.ok(ApiResponse.success(buyApprovalForm));
    }

    // M0004,BL2001,DA2001
    @PostMapping("/approvalList/search")
    public ResponseEntity<ApiResponse<BuyApprovalForm>> search(@RequestBody BuyApprovalForm buyApprovalForm, HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"2".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        List<SizaiModel> sizaiList = buyApprovalService.getDeptRequestList(buyApprovalForm.getStartDate(), buyApprovalForm.getEndDate(), user.getDepid());
        if (sizaiList == null || sizaiList.isEmpty()) {

            ResponseEntity.badRequest().body(ApiResponse.fail("指定期間の購入依頼が登録されていません"));
        } else {
            buyApprovalForm.setSizaiList(sizaiList);
        }
        return ResponseEntity.ok(ApiResponse.success(buyApprovalForm));
    }

    // M0004,BL2003,DA2003
    @PostMapping("/approvalList/approval")
    public ResponseEntity<ApiResponse<BuyApprovalForm>> approval(@RequestBody BuyApprovalForm buyApprovalForm, HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"2".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        boolean approval = buyApprovalService.approval(buyApprovalForm.getTargeId());

        if (approval) {
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            String startDateStr = LocalDate.now().minusMonths(1).format(formatter);
            String endDateStr = LocalDate.now().format(formatter);

            List<SizaiModel> sizaiList = buyApprovalService.getDeptRequestList(startDateStr, endDateStr, user.getDepid());
            buyApprovalForm.setSizaiList(sizaiList);
            return ResponseEntity.ok(ApiResponse.success(buyApprovalForm));
        } else {
            return ResponseEntity.badRequest().body(ApiResponse.fail("承認できませんでした。"));
        }
    }

    // M0004,BL2004,DA2004
    @PostMapping("/approvalList/reject")
    public ResponseEntity<ApiResponse<BuyApprovalForm>> reject(@RequestBody BuyApprovalForm buyApprovalForm, HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"2".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        boolean reject = buyApprovalService.reject(buyApprovalForm.getTargeId());
        if (reject) {
            String startDateStr = buyApprovalForm.getStartDate();
            String endDateStr = buyApprovalForm.getEndDate();
            if (startDateStr == null || endDateStr == null) {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                startDateStr = LocalDate.now().minusMonths(1).format(formatter);
                endDateStr = LocalDate.now().format(formatter);
            }

            List<SizaiModel> sizaiList = buyApprovalService.getDeptRequestList(startDateStr, endDateStr, user.getDepid());
            buyApprovalForm.setSizaiList(sizaiList);
            return ResponseEntity.ok(ApiResponse.success(buyApprovalForm));
        } else {
            return ResponseEntity.badRequest().body(ApiResponse.fail("却下できませんでした。"));
        }
    }

    // M0004,BL2002,DA2002
    @GetMapping("/approvalDetail/init")
    public ResponseEntity<ApiResponse<BuyApprovalForm>> detail(@RequestParam("targetId") int targetId,HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"2".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        BuyApprovalForm buyApprovalForm = new BuyApprovalForm();
        buyApprovalForm.setTargeId(targetId);
        buyApprovalForm.setSizai(buyApprovalService.getRequest(targetId));
        return ResponseEntity.ok(ApiResponse.success(buyApprovalForm));
    }
}