package co.jp.mamol.myapp.controller;

import co.jp.mamol.myapp.common.ApiResponse;
import co.jp.mamol.myapp.model.UserModel;
import co.jp.mamol.myapp.form.LoginForm;
import co.jp.mamol.myapp.service.LoginService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpSession;

import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/api")
public class LoginController {

	@Autowired
	private LoginService loginService;

	@PostMapping("/login")
	public ResponseEntity<ApiResponse<Map<String, Object>>> login(@RequestBody LoginForm loginForm, HttpServletRequest request) {
		HttpSession session = request.getSession();

		if (loginForm.getId() == null || loginForm.getId().isEmpty()) {
			return ResponseEntity.badRequest().body(ApiResponse.<Map<String, Object>>fail("ユーザーIDを入力してください。"));
		} else if (loginForm.getPassword() == null || loginForm.getPassword().isEmpty()) {
			return ResponseEntity.badRequest().body(ApiResponse.<Map<String, Object>>fail("パスワードを入力してください。"));
		}

		UserModel userModel = loginService.findUser(loginForm.getId());
		if (userModel == null || !userModel.getPassword().equals(loginForm.getPassword())) {
			return ResponseEntity.badRequest().body(ApiResponse.<Map<String, Object>>fail("パスワードまたはユーザーIDが間違っています。"));
		}

		session.setAttribute("loginInfo", userModel);

		String redirectUrl = switch (userModel.getRole_class()) {
			case "1" -> "/request";
			case "2" -> "/approval";
			case "3" -> "/order";
			case "4" -> "/inStore";
			case "5" -> "/systemAdmin";
			default -> "/error";
		};

		Map<String, Object> result = new HashMap<>();
		result.put("redirect", redirectUrl);
		result.put("user", userModel);
		return ResponseEntity.ok(ApiResponse.success(result));
	}

	@GetMapping("/logout")
	public ResponseEntity<ApiResponse<Void>> logout(HttpServletRequest request) {
		HttpSession session = request.getSession();
		session.invalidate();
		return ResponseEntity.ok(ApiResponse.success(null));
	}

	@GetMapping("/me")
	public ResponseEntity<ApiResponse<UserModel>> getCurrentUser(HttpServletRequest request) {
		HttpSession session = request.getSession(false);
		String accept = request.getHeader("Accept");

		if (accept == null || !accept.contains("application/json")) {
			return ResponseEntity.status(403).body(ApiResponse.fail("許可されていません"));
		}

		if (session == null) {
			return ResponseEntity.status(401).body(ApiResponse.fail("セッションが無効です"));
		}

		UserModel user = (UserModel) session.getAttribute("loginInfo");
		if (user == null) {
			return ResponseEntity.status(401).body(ApiResponse.fail("ログインが必要です"));
		}

		return ResponseEntity.ok(ApiResponse.success(user));
	}
}