package co.jp.mamol.myapp.controller;

import co.jp.mamol.myapp.common.ApiResponse;
import co.jp.mamol.myapp.form.StoreForm;
import co.jp.mamol.myapp.model.SizaiModel;
import co.jp.mamol.myapp.model.SoukoModel;
import co.jp.mamol.myapp.model.UserModel;
import co.jp.mamol.myapp.service.StoreService;
import jakarta.servlet.http.HttpSession;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/api")
public class StoreController {

    private final StoreService storeService;

    @Autowired
    public StoreController(StoreService storeService) {
        this.storeService = storeService;
    }

    private UserModel getLoginUser(HttpSession session) {

        return (UserModel) session.getAttribute("loginInfo");
    }

    @GetMapping("/inStore/init")
    public ResponseEntity<ApiResponse<StoreForm>> inStoreInit(HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"4".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        StoreForm storeForm = new StoreForm();
        return ResponseEntity.ok(ApiResponse.success(storeForm));
    }

    @PostMapping("/inStore/search")
    public ResponseEntity<ApiResponse<StoreForm>> inStoreSearch(@RequestBody StoreForm storeForm, HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"4".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        SizaiModel sizai = new SizaiModel();
        sizai = storeService.getSizaiById(storeForm.getId());
        if (sizai == null || storeForm.getId() == 0) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("資材存在しません。"));
        }

        storeForm.setSizai(sizai);
        List<SoukoModel> soukoList = storeService.getSoukoList();
        storeForm.setSoukoList(soukoList);

        if (soukoList == null || soukoList.size() == 0) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("倉庫リスト取得失敗しました。"));
        }

        return ResponseEntity.ok(ApiResponse.success(storeForm));
    }

    @PostMapping("/inStore/act")
    public ResponseEntity<ApiResponse<StoreForm>> inStoreAct(@RequestBody StoreForm storeForm,HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"4".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        if (!"5".equals(storeForm.getSizai().getStatus())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("資材の状態は「納品済」ではないため、入庫できません。"));
        }else {
            boolean inStoreFlag = storeService.inStoreAct(storeForm.getSizai());
            if (!inStoreFlag) {

                return ResponseEntity.badRequest().body(ApiResponse.fail("入庫失敗しました。"));
            }
        }

        SizaiModel sizai = storeService.getSizaiById(storeForm.getSizai().getId());
        storeForm.setSizai(sizai);
        return ResponseEntity.ok(ApiResponse.success(storeForm));
    }

    @GetMapping("/outStore/init")
    public ResponseEntity<ApiResponse<StoreForm>> outStoreInit(HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"4".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        StoreForm storeForm = new StoreForm();
        return ResponseEntity.ok(ApiResponse.success(storeForm));
    }

    @PostMapping("/outStore/search")
    public ResponseEntity<ApiResponse<StoreForm>> outStoreSearch(@RequestBody StoreForm storeForm, HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"4".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        SizaiModel sizai = new SizaiModel();
        sizai = storeService.getSizaiById(storeForm.getId());
        if (sizai == null || storeForm.getId() == 0) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("資材存在しません。"));
        }

        storeForm.setSizai(sizai);
        List<SoukoModel> soukoList = storeService.getSoukoList();
        storeForm.setSoukoList(soukoList);

        if (soukoList == null || soukoList.size() == 0) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("倉庫リスト取得失敗しました。"));
        }

        return ResponseEntity.ok(ApiResponse.success(storeForm));
    }

    @PostMapping("/outStore/act")
    public ResponseEntity<ApiResponse<StoreForm>> outStoreAct(@RequestBody StoreForm storeForm,HttpSession session) {
        UserModel user = getLoginUser(session);
        if (user == null || !"4".equals(user.getRole_class())) {
            return ResponseEntity.badRequest().body(ApiResponse.fail("ログイン情報が無効です。再度ログインしてください。"));
        }

        if (!"6".equals(storeForm.getSizai().getStatus())) {

            return ResponseEntity.badRequest().body(ApiResponse.fail("資材の状態は「入庫済」ではないため、出庫できません。"));
        } else {

            boolean outStoreFlag = storeService.outStoreAct(storeForm.getSizai());
            if (!outStoreFlag) {

                return ResponseEntity.badRequest().body(ApiResponse.fail("出庫失敗しました。"));
            }
        }

        SizaiModel sizai = storeService.getSizaiById(storeForm.getSizai().getId());
        storeForm.setSizai(sizai);

        return ResponseEntity.ok(ApiResponse.success(storeForm));
    }
}
